import asyncio
from mcp import ClientSession, StdioServerParameters
from mcp.client.stdio import stdio_client
from langchain_mcp_adapters.tools import load_mcp_tools
from langgraph.prebuilt import create_react_agent
from langchain_openai import ChatOpenAI
import logging
from mcp.client.sse import sse_client

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

async def main():
    try:
        # Define server parameters for stdio transport
        async with sse_client("http://localhost:8080/sse") as (read_stream, write_stream):
            async with ClientSession(read_stream, write_stream) as session:
                # Initialize the connection
                await session.initialize()

                # Load MCP tools for LangChain
                logger.info("Loading MCP tools")
                tools = await load_mcp_tools(session)

                # Set up LLM (replace with your model and API key)
                model = ChatOpenAI(
                    model="gpt-4o",
                    api_key="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"  # Replace with your actual API key
                )
                logger.info("Initialized LLM")

                # Create LangChain ReAcT agent
                logger.info("Creating LangChain agent")
                agent = create_react_agent(model, tools)

                # Test query to call tools
                query = "provide me the weather report in CA?"
                logger.info(f"Invoking agent with query: {query}")
                response = await agent.ainvoke({"messages": [{"role": "user", "content": query}]})

                # Extract the response content
                # Assuming response['messages'] is a list of AIMessage objects
                last_message = response['messages'][-1]
                if hasattr(last_message, 'content'):
                    response_content = last_message.content
                else:
                    response_content = str(last_message)  # Fallback for unexpected format
                logger.info(f"Agent response: {response_content}")

    except Exception as e:
        logger.error(f"Error occurred: {str(e)}")
        raise

if __name__ == "__main__":
    asyncio.run(main())


# server.py
from mcp.server.fastmcp import FastMCP

# Create an MCP server
mcp = FastMCP(
    name="add",
    host="0.0.0.0",  # only used for SSE transport (localhost)
    port=8080,  # only used for SSE transport (set this to any port)
)


# Add an addition tool
@mcp.tool()
def add(a: int, b: int):
    """To implement a song using 2 numbers a,b"""
    return a + b




if __name__ == "__main__":
    # Start the server
    mcp.run(transport="stdio")


# Create server parameters for stdio connection
from langchain_mcp_adapters.client import MultiServerMCPClient
from langgraph.prebuilt import create_react_agent
from langchain_openai import ChatOpenAI


from langchain_google_genai import ChatGoogleGenerativeAI


llm = ChatGoogleGenerativeAI(
    model="gemini-2.0-flash-001",
    google_api_key="AIzaSyDjGE1j1tUrRga-d7f5WRGfZkHPZeJryzQ"
  
)



async def main():
   async with MultiServerMCPClient(
    {
        "math": {
            "command": "python",
            # Make sure to update to the full absolute path to your math_server.py file
            "args": ["main.py"],
            "transport": "stdio",
        },
        "weather": {
            # make sure you start your weather server on port 8000
            "url": "http://localhost:8080/sse",
            "transport": "sse",
        }
    }
) as client:
    agent = create_react_agent(llm, client.get_tools())
    math_response = await agent.ainvoke({"messages": "what's (3 + 5) x 12?"})
    # weather_response = await agent.ainvoke({"messages": "provide me the weather alert in CA?"})
    
    print("Math Response:", math_response["messages"][-1].content)
            
if __name__ == "__main__":      
    
    import asyncio
    asyncio.run(main())
        
        
